==> relaxed_poetry-0.4.1-py3-none-any.whl: poetry/repositories/pypi_repository.py <==
import logging
import urllib.parse
from collections import defaultdict
from typing import Dict
from typing import List
from typing import Union

import requests
from cachecontrol.controller import logger as cache_control_logger
from cachy import CacheManager
from html5lib.html5parser import parse
from poetry.core.packages.dependency import Dependency
from poetry.core.packages.package import Package
from poetry.core.packages.utils.link import Link
from poetry.core.semver.helpers import parse_constraint
from poetry.core.semver.version_constraint import VersionConstraint
from poetry.core.semver.version_range import VersionRange
from poetry.core.version.exceptions import InvalidVersion
from poetry.core.version.markers import parse_marker

from poetry.locations import REPOSITORY_CACHE_DIR
from poetry.utils._compat import to_str
from poetry.utils.patterns import wheel_file_re
from .exceptions import PackageNotFound
from .remote_repository import RemoteRepository
from ..console import console
from ..managed_project import ManagedProject
from ..utils import http

cache_control_logger.setLevel(logging.ERROR)

logger = logging.getLogger(__name__)

from poetry.inspection.info import PackageInfo


class PyPiRepository(RemoteRepository):
    CACHE_VERSION = parse_constraint("1.0.0")

    def __init__(
            self,
            url: str = "https://pypi.org/",
            disable_cache: bool = False,
            fallback: bool = True,
    ) -> None:
        super().__init__(url.rstrip("/") + "/simple/")

        self._base_url = url
        self._disable_cache = disable_cache
        self._fallback = fallback

        release_cache_dir = REPOSITORY_CACHE_DIR / "pypi"
        self._cache = CacheManager(
            {
                "default": "releases",
                "serializer": "json",
                "stores": {
                    "releases": {"driver": "file", "path": str(release_cache_dir)},
                    "packages": {"driver": "dict"},
                },
            }
        )


        self._name = "PyPI"


    def find_packages(self, dependency: Dependency) -> List[Package]:
        """
        Find packages on the remote server.
        """
        constraint = dependency.constraint
        if constraint is None:
            constraint = "*"

        if not isinstance(constraint, VersionConstraint):
            constraint = parse_constraint(constraint)

        allow_prereleases = dependency.allows_prereleases()
        if isinstance(constraint, VersionRange):
            if (
                    constraint.max is not None
                    and constraint.max.is_unstable()
                    or constraint.min is not None
                    and constraint.min.is_unstable()
            ):
                allow_prereleases = True

        try:
            info = self.get_package_info(dependency.name)
        except PackageNotFound:
            self._log(
                "No packages found for {} {}".format(dependency.name, str(constraint)),
                level="debug",
            )
            return []

        packages = []
        ignored_pre_release_packages = []

        for version, release in info["releases"].items():
            if not release:
                # Bad release
                self._log(
                    "No release information found for {}-{}, skipping".format(
                        dependency.name, version
                    ),
                    level="debug",
                )
                continue

            try:
                package = Package(info["info"]["name"], version)
            except InvalidVersion:
                self._log(
                    'Unable to parse version "{}" for the {} package, skipping'.format(
                        version, dependency.name
                    ),
                    level="debug",
                )
                continue

            if package.is_prerelease() and not allow_prereleases:
                if constraint.is_any():
                    # we need this when all versions of the package are pre-releases
                    ignored_pre_release_packages.append(package)
                continue

            if not constraint or (constraint and constraint.allows(package.version)):
                packages.append(package)

        self._log(
            "{} packages found for {} {}".format(
                len(packages), dependency.name, str(constraint)
            ),
            level="debug",
        )

        return packages or ignored_pre_release_packages

    def package(
            self,
            name: str,
            version: str,
            project: ManagedProject,
            extras: (Union[list, None]) = None,
    ) -> Package:
        return self.get_release_info(name, version, project).to_package(name=name, extras=extras)

    def search(self, query: str) -> List[Package]:
        results = []

        search = {"q": query}

        response = http.session().get(self._base_url + "search", params=search, **self._request_auth_args)
        content = parse(response.content, namespaceHTMLElements=False)
        for result in content.findall(".//*[@class='package-snippet']"):
            name = result.find("h3/*[@class='package-snippet__name']").text
            version = result.find("h3/*[@class='package-snippet__version']").text

            if not name or not version:
                continue

            description = result.find("p[@class='package-snippet__description']").text
            if not description:
                description = ""

            try:
                result = Package(name, version, description)
                result.description = to_str(description.strip())
                results.append(result)
            except InvalidVersion:
                self._log(
                    'Unable to parse version "{}" for the {} package, skipping'.format(
                        version, name
                    ),
                    level="debug",
                )

        return results

    def get_package_info(self, name: str) -> dict:
        """
        Return the package information given its name.

        The information is returned from the cache if it exists
        or retrieved from the remote server.
        """
        if self._disable_cache:
            return self._get_package_info(name)

        return self._cache.store("packages").remember_forever(
            name, lambda: self._get_package_info(name)
        )

    def _get_package_info(self, name: str) -> dict:
        data = self._get(f"pypi/{name}/json")
        if data is None:
            raise PackageNotFound(f"Package [{name}] not found.")

        return data

    def get_release_info(self, name: str, version: str, project: ManagedProject) -> "PackageInfo":
        """
        Return the release information given a package name and a version.

        The information is returned from the cache if it exists
        or retrieved from the remote server.
        """
        from poetry.inspection.info import PackageInfo

        self._log(f"Getting info for {name} ({version}) from PyPI", "debug")

        json_data = self._get(f"pypi/{name}/{version}/json")
        if json_data is None:
            raise PackageNotFound(f"Package [{name}] not found.")

        info = json_data["info"]

        data = PackageInfo(
            name=info["name"],
            version=info["version"],
            summary=info["summary"],
            platform=info["platform"],
            requires_dist=info["requires_dist"],
            requires_python=info["requires_python"],
            files=info.get("files", []),
            cache_version=str(self.CACHE_VERSION),
        )

        package = data.to_package()
        if package.python_constraint.intersect(project.env.python_constraint).is_empty():
            # this package is not usable for this project - we can stop trying to bring more information
            return data

        try:
            version_info = json_data["releases"][version]
        except KeyError:
            version_info = []

        for file_info in version_info:
            data.files.append(
                {
                    "file": file_info["filename"],
                    "hash": "sha256:" + file_info["digests"]["sha256"],
                }
            )

        if self._fallback and data.requires_dist is None:
            self._log("No dependencies found, downloading archives", level="debug")
            # No dependencies set (along with other information)
            # This might be due to actually no dependencies
            # or badly set metadata when uploading
            # So, we need to make sure there is actually no
            # dependencies by introspecting packages
            urls = defaultdict(list)
            for url in json_data["urls"]:
                # Only get sdist and wheels if they exist
                dist_type = url["packagetype"]
                if dist_type not in ["sdist", "bdist_wheel"]:
                    continue

                urls[dist_type].append(url["url"])

            if not urls:
                return data

            info = self._get_info_from_urls(urls, project)

            data.requires_dist = info.requires_dist

            if not data.requires_python:
                data.requires_python = info.requires_python

        return data

    def find_links_for_package(self, package: Package) -> List[Link]:
        json_data = self._get(f"pypi/{package.name}/{package.version}/json")
        if json_data is None:
            return []

        links = []
        for url in json_data["urls"]:
            h = "sha256={}".format(url["digests"]["sha256"])
            links.append(Link(url["url"] + "#" + h))

        return links

    def _get(self, endpoint: str) -> Union[dict, None]:

        session = http.cached_session(True)

        try:
            json_response = session.get(self._base_url + endpoint)
        except requests.exceptions.TooManyRedirects:
            # Cache control redirect loop.
            # We try to remove the cache and try again
            http.remove_persistent_cache(self._base_url + endpoint)
            json_response = session.get(self._base_url + endpoint)

        if json_response.status_code == 404:
            return None

        json_data = json_response.json()

        return json_data

    def _get_info_from_urls(
            self,
            urls: Dict[str, List[str]],
            project: ManagedProject,
    ) -> "PackageInfo":
        # Checking wheels first as they are more likely to hold
        # the necessary information
        if "bdist_wheel" in urls:
            # Check for a universal wheel
            wheels = urls["bdist_wheel"]

            universal_wheel = None
            universal_python2_wheel = None
            universal_python3_wheel = None
            platform_specific_wheels = []
            platform_accepted_tags = project.env.supported_tags_strset
            for wheel in wheels:
                link = Link(wheel)
                m = wheel_file_re.match(link.filename)
                if not m:
                    continue

                pyver = m.group("pyver")
                abi = m.group("abi")
                plat = m.group("plat")
                if abi == "none" and plat == "any":
                    # Universal wheel
                    if pyver == "py2.py3":
                        # Any Python
                        universal_wheel = wheel
                    elif pyver == "py2":
                        universal_python2_wheel = wheel
                    else:
                        universal_python3_wheel = wheel
                else:
                    if f"{pyver}-{abi}-{plat}" in platform_accepted_tags:
                        platform_specific_wheels.append(wheel)

            if universal_wheel is not None:
                return self._get_info_from_wheel(universal_wheel, project)

            info = None
            if universal_python2_wheel and universal_python3_wheel:
                info = self._get_info_from_wheel(universal_python2_wheel, project)

                py3_info = self._get_info_from_wheel(universal_python3_wheel, project)
                if py3_info.requires_dist:
                    if not info.requires_dist:
                        info.requires_dist = py3_info.requires_dist

                        return info

                    py2_requires_dist = {
                        Dependency.create_from_pep_508(r).to_pep_508()
                        for r in info.requires_dist
                    }
                    py3_requires_dist = {
                        Dependency.create_from_pep_508(r).to_pep_508()
                        for r in py3_info.requires_dist
                    }
                    base_requires_dist = py2_requires_dist & py3_requires_dist
                    py2_only_requires_dist = py2_requires_dist - py3_requires_dist
                    py3_only_requires_dist = py3_requires_dist - py2_requires_dist

                    # Normalizing requires_dist
                    requires_dist = list(base_requires_dist)
                    for requirement in py2_only_requires_dist:
                        dep = Dependency.create_from_pep_508(requirement)
                        dep.marker = dep.marker.intersect(
                            parse_marker("python_version == '2.7'")
                        )
                        requires_dist.append(dep.to_pep_508())

                    for requirement in py3_only_requires_dist:
                        dep = Dependency.create_from_pep_508(requirement)
                        dep.marker = dep.marker.intersect(
                            parse_marker("python_version >= '3'")
                        )
                        requires_dist.append(dep.to_pep_508())

                    info.requires_dist = sorted(list(set(requires_dist)))

            if info:
                return info

            # Prefer non platform specific wheels
            if universal_python3_wheel:
                return self._get_info_from_wheel(universal_python3_wheel, project)

            if universal_python2_wheel:
                return self._get_info_from_wheel(universal_python2_wheel, project)

            if platform_specific_wheels:
                # Pick the first wheel available and hope for the best
                return self._get_info_from_wheel(platform_specific_wheels[0], project)

        return self._get_info_from_sdist(urls["sdist"][0], project)

    def _get_info_from_wheel(self, url: str, project: ManagedProject) -> "PackageInfo":

        def get_info():
            from poetry.inspection.info import PackageInfo

            self._log(
                "Downloading wheel: {}".format(
                    urllib.parse.urlparse(url).path.rsplit("/")[-1]
                ),
                level="debug",
            )

            # filename = os.path.basename(urllib.parse.urlparse(url).path.rsplit("/")[-1])
            from poetry.app.relaxed_poetry import rp
            wheel_archive = rp.artifacts.fetch(Link(url), authenticator=project.authenticator, io=console)

            return PackageInfo.from_wheel(wheel_archive).asdict()

        return PackageInfo.load(self._cache.remember_forever(f"wheels/{url}", get_info))


    def _get_info_from_sdist(self, url: str, project: ManagedProject) -> "PackageInfo":

        def get_info():
            from poetry.inspection.info import PackageInfo

            self._log(
                "Downloading sdist: {}".format(
                    urllib.parse.urlparse(url).path.rsplit("/")[-1]
                ),
                level="debug",
            )

            # filename = os.path.basename(urllib.parse.urlparse(url).path)

            from poetry.app.relaxed_poetry import rp
            wheel_archive = rp.artifacts.fetch(Link(url), authenticator=project.authenticator, io=console)
            return PackageInfo.from_sdist(wheel_archive).asdict()

        return PackageInfo.load(self._cache.remember_forever(f"sdists/{url}", get_info))

    def _log(self, msg: str, level: str = "info") -> None:
        getattr(logger, level)(f"<debug>{self._name}:</debug> {msg}")

==> relaxed_poetry-0.5.0-py3-none-any.whl: poetry/repositories/pypi_repository.py <==
import logging
import urllib.parse
from collections import defaultdict
from typing import Dict
from typing import List
from typing import Union

import requests
from cachecontrol.controller import logger as cache_control_logger
from cachy import CacheManager
from html5lib.html5parser import parse
from poetry.core.packages.dependency import Dependency
from poetry.core.packages.package import Package
from poetry.core.packages.utils.link import Link
from poetry.core.semver.helpers import parse_constraint
from poetry.core.semver.version_constraint import VersionConstraint
from poetry.core.semver.version_range import VersionRange
from poetry.core.version.exceptions import InvalidVersion
from poetry.core.version.markers import parse_marker

from poetry.locations import REPOSITORY_CACHE_DIR
from poetry.utils._compat import to_str
from poetry.utils.patterns import wheel_file_re
from .exceptions import PackageNotFound
from .remote_repository import RemoteRepository
from ..console import console
from ..managed_project import ManagedProject
from ..utils import http

cache_control_logger.setLevel(logging.ERROR)

logger = logging.getLogger(__name__)

from poetry.inspection.info import PackageInfo


class PyPiRepository(RemoteRepository):
    CACHE_VERSION = parse_constraint("1.0.0")

    def __init__(
            self,
            url: str = "https://pypi.org/",
            disable_cache: bool = False,
            fallback: bool = True,
    ) -> None:
        super().__init__(url.rstrip("/") + "/simple/")

        self._base_url = url
        self._disable_cache = disable_cache
        self._fallback = fallback

        release_cache_dir = REPOSITORY_CACHE_DIR / "pypi"
        self._cache = CacheManager(
            {
                "default": "releases",
                "serializer": "json",
                "stores": {
                    "releases": {"driver": "file", "path": str(release_cache_dir)},
                    "packages": {"driver": "dict"},
                },
            }
        )

        self._name = "PyPI"

    def find_packages(self, dependency: Dependency) -> List[Package]:
        """
        Find packages on the remote server.
        """
        constraint = dependency.constraint
        if constraint is None:
            constraint = "*"

        if not isinstance(constraint, VersionConstraint):
            constraint = parse_constraint(constraint)

        allow_prereleases = dependency.allows_prereleases()
        if isinstance(constraint, VersionRange):
            if (
                    constraint.max is not None
                    and constraint.max.is_unstable()
                    or constraint.min is not None
                    and constraint.min.is_unstable()
            ):
                allow_prereleases = True

        try:
            info = self.get_package_info(dependency.name)
        except PackageNotFound:
            self._log(
                "No packages found for {} {}".format(dependency.name, str(constraint)),
                level="debug",
            )
            return []

        packages = []
        ignored_pre_release_packages = []

        for version, release in info["releases"].items():
            if not release:
                # Bad release
                self._log(
                    "No release information found for {}-{}, skipping".format(
                        dependency.name, version
                    ),
                    level="debug",
                )
                continue

            try:
                package = Package(info["info"]["name"], version)
            except InvalidVersion:
                self._log(
                    'Unable to parse version "{}" for the {} package, skipping'.format(
                        version, dependency.name
                    ),
                    level="debug",
                )
                continue

            if package.is_prerelease() and not allow_prereleases:
                if constraint.is_any():
                    # we need this when all versions of the package are pre-releases
                    ignored_pre_release_packages.append(package)
                continue

            if not constraint or (constraint and constraint.allows(package.version)):
                packages.append(package)

        self._log(
            "{} packages found for {} {}".format(
                len(packages), dependency.name, str(constraint)
            ),
            level="debug",
        )

        return packages or ignored_pre_release_packages

    def package(
            self,
            name: str,
            version: str,
            project: ManagedProject,
            extras: (Union[list, None]) = None,
    ) -> Package:
        return self.get_release_info(name, version, project).to_package(name=name, extras=extras)

    def search(self, query: str) -> List[Package]:
        results = []

        search = {"q": query}

        response = http.session().get(self._base_url + "search", params=search)
        content = parse(response.content, namespaceHTMLElements=False)
        for result in content.findall(".//*[@class='package-snippet']"):
            name = result.find("h3/*[@class='package-snippet__name']").text
            version = result.find("h3/*[@class='package-snippet__version']").text

            if not name or not version:
                continue

            description = result.find("p[@class='package-snippet__description']").text
            if not description:
                description = ""

            try:
                result = Package(name, version, description)
                result.description = to_str(description.strip())
                results.append(result)
            except InvalidVersion:
                self._log(
                    'Unable to parse version "{}" for the {} package, skipping'.format(
                        version, name
                    ),
                    level="debug",
                )

        return results

    def get_package_info(self, name: str) -> dict:
        """
        Return the package information given its name.

        The information is returned from the cache if it exists
        or retrieved from the remote server.
        """
        if self._disable_cache:
            return self._get_package_info(name)

        return self._cache.store("packages").remember_forever(
            name, lambda: self._get_package_info(name)
        )

    def _get_package_info(self, name: str) -> dict:
        data = self._get(f"pypi/{name}/json")
        if data is None:
            raise PackageNotFound(f"Package [{name}] not found.")

        return data

    def get_release_info(self, name: str, version: str, project: ManagedProject) -> "PackageInfo":
        """
        Return the release information given a package name and a version.

        The information is returned from the cache if it exists
        or retrieved from the remote server.
        """
        from poetry.inspection.info import PackageInfo

        self._log(f"Getting info for {name} ({version}) from PyPI", "debug")

        json_data = self._get(f"pypi/{name}/{version}/json")
        if json_data is None:
            raise PackageNotFound(f"Package [{name}] not found.")

        info = json_data["info"]

        data = PackageInfo(
            name=info["name"],
            version=info["version"],
            summary=info["summary"],
            platform=info["platform"],
            requires_dist=info["requires_dist"],
            requires_python=info["requires_python"],
            files=info.get("files", []),
            cache_version=str(self.CACHE_VERSION),
        )

        package = data.to_package()
        if package.python_constraint.intersect(project.env.python_constraint).is_empty():
            # this package is not usable for this project - we can stop trying to bring more information
            return data

        try:
            version_info = json_data["releases"][version]
        except KeyError:
            version_info = []

        for file_info in version_info:
            data.files.append(
                {
                    "file": file_info["filename"],
                    "hash": "sha256:" + file_info["digests"]["sha256"],
                }
            )

        if self._fallback and data.requires_dist is None:
            self._log("No dependencies found, downloading archives", level="debug")
            # No dependencies set (along with other information)
            # This might be due to actually no dependencies
            # or badly set metadata when uploading
            # So, we need to make sure there is actually no
            # dependencies by introspecting packages
            urls = defaultdict(list)
            for url in json_data["urls"]:
                # Only get sdist and wheels if they exist
                dist_type = url["packagetype"]
                if dist_type not in ["sdist", "bdist_wheel"]:
                    continue

                urls[dist_type].append(url["url"])

            if not urls:
                return data

            info = self._get_info_from_urls(urls, project)

            data.requires_dist = info.requires_dist

            if not data.requires_python:
                data.requires_python = info.requires_python

        return data

    def find_links_for_package(self, package: Package) -> List[Link]:
        json_data = self._get(f"pypi/{package.name}/{package.version}/json")
        if json_data is None:
            return []

        links = []
        for url in json_data["urls"]:
            h = "sha256={}".format(url["digests"]["sha256"])
            links.append(Link(url["url"] + "#" + h))

        return links

    def _get(self, endpoint: str) -> Union[dict, None]:

        session = http.cached_session(True)

        try:
            json_response = session.get(self._base_url + endpoint)
        except requests.exceptions.TooManyRedirects:
            # Cache control redirect loop.
            # We try to remove the cache and try again
            http.remove_persistent_cache(self._base_url + endpoint)
            json_response = session.get(self._base_url + endpoint)

        if json_response.status_code == 404:
            return None

        json_data = json_response.json()

        return json_data

    def _get_info_from_urls(
            self,
            urls: Dict[str, List[str]],
            project: ManagedProject,
    ) -> "PackageInfo":
        # Checking wheels first as they are more likely to hold
        # the necessary information
        if "bdist_wheel" in urls:
            # Check for a universal wheel
            wheels = urls["bdist_wheel"]

            universal_wheel = None
            universal_python2_wheel = None
            universal_python3_wheel = None
            platform_specific_wheels = []
            platform_accepted_tags = project.env.supported_tags_strset
            for wheel in wheels:
                link = Link(wheel)
                m = wheel_file_re.match(link.filename)
                if not m:
                    continue

                pyver = m.group("pyver")
                abi = m.group("abi")
                plat = m.group("plat")
                if abi == "none" and plat == "any":
                    # Universal wheel
                    if pyver == "py2.py3":
                        # Any Python
                        universal_wheel = wheel
                    elif pyver == "py2":
                        universal_python2_wheel = wheel
                    else:
                        universal_python3_wheel = wheel
                else:
                    if f"{pyver}-{abi}-{plat}" in platform_accepted_tags:
                        platform_specific_wheels.append(wheel)

            if universal_wheel is not None:
                return self._get_info_from_wheel(universal_wheel, project)

            info = None
            if universal_python2_wheel and universal_python3_wheel:
                info = self._get_info_from_wheel(universal_python2_wheel, project)

                py3_info = self._get_info_from_wheel(universal_python3_wheel, project)
                if py3_info.requires_dist:
                    if not info.requires_dist:
                        info.requires_dist = py3_info.requires_dist

                        return info

                    py2_requires_dist = {
                        Dependency.create_from_pep_508(r).to_pep_508()
                        for r in info.requires_dist
                    }
                    py3_requires_dist = {
                        Dependency.create_from_pep_508(r).to_pep_508()
                        for r in py3_info.requires_dist
                    }
                    base_requires_dist = py2_requires_dist & py3_requires_dist
                    py2_only_requires_dist = py2_requires_dist - py3_requires_dist
                    py3_only_requires_dist = py3_requires_dist - py2_requires_dist

                    # Normalizing requires_dist
                    requires_dist = list(base_requires_dist)
                    for requirement in py2_only_requires_dist:
                        dep = Dependency.create_from_pep_508(requirement)
                        dep.marker = dep.marker.intersect(
                            parse_marker("python_version == '2.7'")
                        )
                        requires_dist.append(dep.to_pep_508())

                    for requirement in py3_only_requires_dist:
                        dep = Dependency.create_from_pep_508(requirement)
                        dep.marker = dep.marker.intersect(
                            parse_marker("python_version >= '3'")
                        )
                        requires_dist.append(dep.to_pep_508())

                    info.requires_dist = sorted(list(set(requires_dist)))

            if info:
                return info

            # Prefer non platform specific wheels
            if universal_python3_wheel:
                return self._get_info_from_wheel(universal_python3_wheel, project)

            if universal_python2_wheel:
                return self._get_info_from_wheel(universal_python2_wheel, project)

            if platform_specific_wheels:
                # Pick the first wheel available and hope for the best
                return self._get_info_from_wheel(platform_specific_wheels[0], project)

        return self._get_info_from_sdist(urls["sdist"][0], project)

    def _get_info_from_wheel(self, url: str, project: ManagedProject) -> "PackageInfo":

        def get_info():
            from poetry.inspection.info import PackageInfo

            self._log(
                "Downloading wheel: {}".format(
                    urllib.parse.urlparse(url).path.rsplit("/")[-1]
                ),
                level="debug",
            )

            # filename = os.path.basename(urllib.parse.urlparse(url).path.rsplit("/")[-1])
            from poetry.app.relaxed_poetry import rp
            wheel_archive = rp.artifacts.fetch(Link(url), authenticator=project.authenticator, io=console)

            return PackageInfo.from_wheel(wheel_archive).asdict()

        return PackageInfo.load(self._cache.remember_forever(f"wheels/{url}", get_info))

    def _get_info_from_sdist(self, url: str, project: ManagedProject) -> "PackageInfo":

        def get_info():
            from poetry.inspection.info import PackageInfo

            self._log(
                "Downloading sdist: {}".format(
                    urllib.parse.urlparse(url).path.rsplit("/")[-1]
                ),
                level="debug",
            )

            # filename = os.path.basename(urllib.parse.urlparse(url).path)

            from poetry.app.relaxed_poetry import rp
            wheel_archive = rp.artifacts.fetch(Link(url), authenticator=project.authenticator, io=console)
            return PackageInfo.from_sdist(wheel_archive).asdict()

        return PackageInfo.load(self._cache.remember_forever(f"sdists/{url}", get_info))

    def _log(self, msg: str, level: str = "info") -> None:
        getattr(logger, level)(f"<debug>{self._name}:</debug> {msg}")
